# -*- coding: utf-8 -*-


from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers
from aparnik.api.serializers import ModelSerializer
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.settings import aparnik_settings
from aparnik.utils.utils import is_app_installed
from aparnik.contrib.reviews.models import Review, Like, BaseReview, ReviewSummary
from aparnik.contrib.basemodels.models import BaseModel
from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer, BaseModelListSerializer

UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


# Base Review List Serializer
class BaseReviewListSerializer(BaseModelListSerializer):
    url_like = serializers.SerializerMethodField()
    url_dislike = serializers.SerializerMethodField()
    url_action_like = serializers.SerializerMethodField()
    url_action_dislike = serializers.SerializerMethodField()
    url_action_base_review_status_set = serializers.SerializerMethodField()
    like_count = serializers.SerializerMethodField()
    dislike_count = serializers.SerializerMethodField()
    user = serializers.SerializerMethodField()
    childs = serializers.SerializerMethodField()
    is_like = serializers.SerializerMethodField()
    is_dislike = serializers.SerializerMethodField()
    has_permit_change_status = serializers.SerializerMethodField()
    has_permit_replay = serializers.SerializerMethodField()
    has_permit_edit = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(BaseReviewListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BaseReview
        fields = BaseModelListSerializer.Meta.fields + [
            'title',
            'description',
            'url_like',
            'url_dislike',
            'url_action_like',
            'url_action_dislike',
            'url_action_base_review_status_set',
            'like_count',
            'dislike_count',
            'user',
            'is_published',
            'created_at',
            'childs',
            'has_permit_change_status',
            'has_permit_replay',
            'has_permit_edit',
            'is_like',
            'is_dislike',
        ]
        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + ['like_count', 'dislike_count', 'childs',
                                                                            'is_like',
                                                                            'is_dislike',
                                                                            ]

    def get_url_like(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_like_uri())

    def get_url_dislike(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_dislike_uri())

    def get_url_action_like(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_action_like())

    def get_url_action_dislike(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_action_dislike())

    def get_like_count(self, obj):
        return Like.objects.likes_count(review_obj=obj.id)

    def get_dislike_count(self, obj):
        return Like.objects.dislikes_count(review_obj=obj.id)

    def get_user(self, obj):
        return UserSummeryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data

    def get_childs(self, obj):
        return BaseReviewListPolymorphicSerializer(obj.child.all()[:2], many=True, read_only=True, context=self.context).data if obj.child.all() else None

    def get_url_action_base_review_status_set(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_action_base_review_status_set())

    def get_has_permit_change_status(self, obj):
        user = self.context['request'].user
        return obj.has_permit_change_status(user)

    def get_has_permit_replay(self, obj):
        user = self.context['request'].user
        return obj.has_permit_replay(user)

    def get_has_permit_edit(self, obj):
        user = self.context['request'].user
        return obj.has_permit_edit(user)

    def get_is_like(self, obj):
        user = self.context['request'].user
        return obj.is_like(user)

    def get_is_dislike(self, obj):
        user = self.context['request'].user
        return obj.is_dislike(user)



# Base Review Details Serializer
class BaseReviewDetailsSerializer(BaseReviewListSerializer, BaseModelDetailSerializer):
    model = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(BaseReviewDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BaseReview
        fields = BaseReviewListSerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields + [
            'model',
        ]
        read_only_fields = BaseReviewListSerializer.Meta.read_only_fields + BaseModelDetailSerializer.Meta.read_only_fields + ['is_published']

    # def get_url(self, obj):
    #     return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_model(self, obj):
        from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
        return ModelListPolymorphicSerializer(obj.model_obj.get_real_instance(), many=False, read_only=True, context=self.context).data


class ReviewSummaryListSerializer(ModelSerializer):

    def __init__(self, *args, **kwargs):
        super(ReviewSummaryListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = ReviewSummary
        fields = [
            'id',
            'rate',
            'count',
            'percentage',
        ]
        read_only_fields = []


# Review List Serializer
class ReviewListSerializer(BaseReviewListSerializer):

    def __init__(self, *args, **kwargs):
        super(ReviewListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Review
        fields = BaseReviewListSerializer.Meta.fields + [
            'rate'
        ]
        read_only_fields = BaseReviewListSerializer.Meta.read_only_fields + []


# Review Details Serializer
class ReviewDetailsSerializer(ReviewListSerializer, BaseReviewDetailsSerializer):

    def __init__(self, *args, **kwargs):
        super(ReviewDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Review
        fields = ReviewListSerializer.Meta.fields + BaseReviewDetailsSerializer.Meta.fields + [

        ]
        read_only_fields = ReviewListSerializer.Meta.read_only_fields + BaseReviewDetailsSerializer.Meta.read_only_fields + []


# Review Create Serializer
class ReviewCreateSerializer(ReviewDetailsSerializer):
    model_obj_id = serializers.IntegerField(write_only=True, required=True)
    parent_obj_id = serializers.IntegerField(write_only=True, required=False)
    resourcetype = serializers.SerializerMethodField()

    class Meta:
        model = Review
        fields = ReviewDetailsSerializer.Meta.fields + [
            'model_obj_id',
            'parent_obj_id',
            'resourcetype',
        ]
        read_only_fields = ReviewDetailsSerializer.Meta.read_only_fields + []

    def validate_rate(self, value):
        if value < 1 or value > 5:
            raise serializers.ValidationError(_("The rate should be between 1 and 5"))
        return value

    def validate_model_obj_id(self, value):
        try:
            basemodel = get_object_or_404(BaseModel.objects.all(), id=value)
            return value
        except Exception as e:
            raise serializers.ValidationError(_("Base Model not found"))

    def validate_parent_obj_id(self, value):
        try:
            review = get_object_or_404(Review.objects.all(), id=value)
            return value
        except Exception as e:
            raise serializers.ValidationError(_("Review Parent not found"))

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_resourcetype(self, obj):
        return 'Review'


# Like List Serializer
class LikeListSerializer(ModelSerializer):
    url = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(LikeListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Like
        fields = [
            'id',
            'url',
            'is_like',
        ]

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())


# Like Details Serializer
class LikeDetailsSerializer(LikeListSerializer):
    user = serializers.SerializerMethodField()
    review = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(LikeDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Like
        fields = LikeListSerializer.Meta.fields + [
            'user',
            'review',
        ]

    def get_user(self, obj):
        return UserSummeryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data

    def get_review(self, obj):
        from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
        return ModelListPolymorphicSerializer(obj.review_obj, many=False, read_only=True, context=self.context).data


# Review List Polymorphic
class BaseReviewListPolymorphicSerializer(PolymorphicSerializer):

    def __init__(self, *args, **kwargs):
        from aparnik.contrib.questionanswers.api.serializers import QAListSerializer
        from aparnik.contrib.questionanswers.models import QA

        self.model_serializer_mapping = {
            BaseReview: BaseReviewListSerializer,
            Review: ReviewListSerializer,
            QA: QAListSerializer
        }
        super(BaseReviewListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


# Review Details Polymorphic
class BaseReviewDetailsPolymorphicSerializer(PolymorphicSerializer):

    def __init__(self, *args, **kwargs):
        from aparnik.contrib.questionanswers.api.serializers import QADetailSerializer
        from aparnik.contrib.questionanswers.models import QA

        self.model_serializer_mapping = {
            BaseReview: BaseReviewDetailsSerializer,
            Review: ReviewDetailsSerializer,
            QA: QADetailSerializer
        }
        super(BaseReviewDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.api.serializers import BaseSegmentListSerializer, BaseSegmentDetailSerializer
    from ..models import BaseReviewSegment

    # Base Segment ListSegmentReview
    class SegmentBaseReviewListSerializer(BaseSegmentListSerializer):

        class Meta:
            model = BaseReviewSegment
            fields = BaseSegmentListSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentListSerializer.Meta.read_only_fields + []


    # Book Segment Details
    class SegmentBaseReviewDetailSerializer(BaseSegmentDetailSerializer, SegmentBaseReviewListSerializer):

        class Meta:
            model = BaseReviewSegment
            fields = SegmentBaseReviewListSerializer.Meta.fields + BaseSegmentDetailSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentDetailSerializer.Meta.read_only_fields + SegmentBaseReviewListSerializer.Meta.read_only_fields + []


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-20 10:55


import aparnik.utils.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('filefields', '0004_filefield_title'),
        ('socials', '0001_initial'),
        ('shortblogs', '0001_initial'),
        ('sliders', '0004_sliderimage'),
    ]

    operations = [
        migrations.CreateModel(
            name='Information',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('website', models.URLField(max_length=255, verbose_name='Website')),
                ('address', models.CharField(max_length=255, verbose_name='Address')),
                ('phone', aparnik.utils.fields.PhoneField(max_length=255, validators=[django.core.validators.RegexValidator(code=b'nomatch', message='phone is not valid, please insert with code', regex=b'^0(?!0)\\d{2}([0-9]{8})$')], verbose_name='Phone Number')),
                ('email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='Email')),
                ('about_us', models.TextField(verbose_name='About Us')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
                ('image', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='filefields.FileField', verbose_name='Image')),
                ('short_blogs', models.ManyToManyField(to='shortblogs.ShortBlog', verbose_name='Content Segment')),
                ('slider_segment_obj', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='aboutus_sliders', to='sliders.SliderSegment', verbose_name='Slider Segment')),
                ('socials', models.ManyToManyField(to='socials.SocialNetwork', verbose_name='Socials')),
            ],
            options={
                'verbose_name': 'Information',
                'verbose_name_plural': 'Informations',
            },
        ),
    ]



from django.urls import reverse
from rest_framework import serializers

from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer, BaseModelListSerializer, ModelListPolymorphicSerializer
from aparnik.contrib.province.api.serializers import CityDetailSerializer
from ..models import BankAccount, BankName


# List Serializer
class BankNameListSerializer(BaseModelListSerializer):

    def __init__(self, *args, **kwargs):
        super(BankNameListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BankName
        fields = BaseModelListSerializer.Meta.fields + [
            'title',
        ]

        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + [
        ]


# Details Serializer
class BankNameDetailSerializer(BaseModelDetailSerializer, BankNameListSerializer):

    def __init__(self, *args, **kwargs):
        super(BankNameDetailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BankName
        fields = BaseModelDetailSerializer.Meta.fields + BankNameListSerializer.Meta.fields + [
        ]

        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + BankNameListSerializer.Meta.read_only_fields + []


class BankAccountListSerializer(BaseModelListSerializer):

    bank_name = serializers.SerializerMethodField()
    url_set_default = serializers.SerializerMethodField()
    url_update = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(BankAccountListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BankAccount
        fields = BaseModelListSerializer.Meta.fields + [
            'url_update',
            'bank_name',
            'account_number',
            'shaba_number',
            'card_number',
            'is_default',
            'url_set_default',
            'resourcetype'
        ]

        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + [
            'url_update',
            'bank_name',
            'is_default',
            'url_set_default',
            'resourcetype'
        ]

    def get_url_update(self, obj):
        return self.context['request'].build_absolute_uri(reverse('aparnik-api:bankaccounts:update', args=[obj.id]))

    def get_bank_name(self, obj):
        return ModelListPolymorphicSerializer(obj.bank_name_obj, many=False, read_only=True, context=self.context).data

    def get_url_set_default(self, obj):
        return self.context['request'].build_absolute_uri(reverse('aparnik-api:bankaccounts:set', args=[obj.id]))

    def get_resourcetype(self, obj):
        return 'BankAccount'


# Details Serializer
class BankAccountDetailSerializer(BaseModelDetailSerializer, BankAccountListSerializer):
    bank_name_obj_id = serializers.IntegerField(required=True, write_only=True)

    def __init__(self, *args, **kwargs):
        super(BankAccountDetailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BankAccount
        fields = BaseModelDetailSerializer.Meta.fields + BankAccountListSerializer.Meta.fields + [
            'bank_name_obj_id',
        ]

        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + BankAccountListSerializer.Meta.read_only_fields + []



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-02-11 14:16


from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0011_auto_20181202_1529'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now, verbose_name='Created at'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='user',
            name='update_at',
            field=models.DateTimeField(auto_now=True, verbose_name='Update at'),
        ),
    ]



import os
import re

def read_env():
    """Pulled from Honcho code with minor updates, reads local default
    environment variables from a .env file located in the project root
    directory.
    """
    try:
        with open('.env') as f:
            content = f.read()
    except IOError:
        content = ''

    for line in content.splitlines():
        m1 = re.match(r'\A([A-Za-z_0-9]+)=(.*)\Z', line)
        if m1:
            key, val = m1.group(1), m1.group(2)

            m2 = re.match(r"\A'(.*)'\Z", val)
            if m2:
                val = m2.group(1)
            m3 = re.match(r'\A"(.*)"\Z', val)
            if m3:
                val = re.sub(r'\\(.)', r'\1', m3.group(1))
            os.environ.setdefault(key, val)

read_env()

from .base import *

try:
    from .local_settings import *

except:
    from .servers.share_servers import *


# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.contrib.users.admin import get_update_at
from .models import Support
from aparnik.contrib.basemodels.admin import BaseModelAdmin
# Register your models here.


class SupportAdmin(BaseModelAdmin):
    fields = ['title', 'online', 'offline', 'phone', 'online_days', 'socials', 'is_active']
    exclude = []
    list_display = ['id', 'title', 'is_active', get_update_at, ]
    search_fields = ['title', ]
    list_filter = ['is_active', 'title']

    def __init__(self, *args, **kwargs):
        Klass = SupportAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields

    class Meta:
        model = Support


admin.site.register(Support, SupportAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-02 19:21


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0003_basemodel_update_needed'),
        ('reviews', '0002_auto_20181025_1543'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReviewSummary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('rate', models.CharField(max_length=1, verbose_name='Rate')),
                ('count', models.BigIntegerField(default=0, verbose_name='Count')),
                ('model', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='review_summaries', to='basemodels.BaseModel', verbose_name='Models')),
            ],
            options={
                'verbose_name': 'Reivew Summary',
                'verbose_name_plural': 'Review Summary',
            },
        ),
    ]



from django.conf.urls import url, include

from .views import VoucherListAPIView


app_name = 'vouchers'

urlpatterns = [
    url(r'^$', VoucherListAPIView.as_view(), name='list'),
    # url(r'^model/(?P<model_id>\d+)/set/$', BookmarkSetAPIView.as_view(), name='set'),

]


from django.conf.urls import url, include
from .views import CoSaleUserListAPIView, CoSaleListAPIView


app_name = 'cosales'

urlpatterns = [
    url(r'^$', CoSaleListAPIView.as_view(), name='list'),
    url(r'^user/$', CoSaleUserListAPIView.as_view(), name='user-list'),
    # url(r'^(?P<id>\d+)/add-address/$', AddOrderAddressAPIView.as_view(), name='add-address'),
]



# Generated by Django 2.2 on 2019-06-15 14:30

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('files', '0007_auto_20190501_1147'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='file',
            options={'ordering': ['-created_at'], 'verbose_name': 'فایل', 'verbose_name_plural': 'فایل ها'},
        ),
        migrations.AlterField(
            model_name='file',
            name='banner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='shop_file_banner', to='filefields.FileField', verbose_name='بنر'),
        ),
        migrations.AlterField(
            model_name='file',
            name='cover',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='shop_file_cover', to='filefields.FileField', verbose_name='تصویر جلد'),
        ),
        migrations.AlterField(
            model_name='file',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='توضیحات'),
        ),
        migrations.AlterField(
            model_name='file',
            name='file_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='shop_file_obj', to='filefields.FileField', verbose_name='فایل'),
        ),
        migrations.AlterField(
            model_name='file',
            name='publish_date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='تاریخ انتشار'),
        ),
    ]



from django.conf.urls import url, include

from .views import ProductSortAPIView, ProductListAPIView

app_name = 'products'

urlpatterns = [
    url(r'^$', ProductListAPIView.as_view(), name='list'),
    url(r'^sort/$', ProductSortAPIView.as_view(), name='sort'),
]





import unicodedata

from django import forms
from django.contrib.auth import (
    authenticate, get_user_model,
)
from django.utils.text import capfirst
from django.utils.translation import ugettext, ugettext_lazy as _

from aparnik.utils.utils import convert_iran_phone_number_to_world_number
from aparnik.settings import aparnik_settings

UserModel = get_user_model()


class UsernameField(forms.CharField):
    def to_python(self, value):
        return unicodedata.normalize('NFKC', super(UsernameField, self).to_python(value))


class AuthenticationForm(forms.Form):
    """
    Base class for authenticating users. Extend this to get a form that accepts
    username/password logins.
    """
    username = UsernameField(
        max_length=254,
        widget=forms.TextInput(attrs={'autofocus': True}),
    )
    password = forms.CharField(
        label=_("Password"),
        strip=False,
        widget=forms.PasswordInput,
    )

    step = 1

    error_messages = {
        'invalid_login': _(
            "Please enter a correct %(username)s and password. Note that both "
            "fields may be case-sensitive."
        ),
        'inactive': _("This account is inactive."),
    }

    def __init__(self, request=None, *args, **kwargs):
        """
        The 'request' parameter is set for custom auth use by subclasses.
        The form data comes in via the standard 'data' kwarg.
        """
        self.request = request
        self.user_cache = None
        super(AuthenticationForm, self).__init__(*args, **kwargs)

        # Set the label for the "username" field.
        self.username_field = UserModel._meta.get_field(UserModel.USERNAME_FIELD)
        if self.fields['username'].label is None:
            self.fields['username'].label = capfirst(self.username_field.verbose_name)

    def clean(self):
        username = self.cleaned_data.get('username')
        username = convert_iran_phone_number_to_world_number(username)

        password = self.cleaned_data.get('password')

        if username is not None and password is None:
            try:
                user = UserModel.objects.get(username=username)
                self.step = 2
                if not aparnik_settings.USER_LOGIN_WITH_PASSWORD:
                    user.OTARequest()

            except:
                raise forms.ValidationError(
                    self.error_messages['invalid_login'],
                    code='invalid_login',
                    params={'username': self.username_field.verbose_name},
                )
        elif username is not None and password is not None:
            self.step = 2
            self.user_cache = authenticate(self.request, username=username, password=password)
            if self.user_cache is None:
                raise forms.ValidationError(
                    self.error_messages['invalid_login'],
                    code='invalid_login',
                    params={'username': self.username_field.verbose_name},
                )
            else:
                self.confirm_login_allowed(self.user_cache)

        return self.cleaned_data

    def confirm_login_allowed(self, user):
        """
        Controls whether the given User may log in. This is a policy setting,
        independent of end-user authentication. This default behavior is to
        allow login by active users, and reject login by inactive users.

        If the given user cannot log in, this method should raise a
        ``forms.ValidationError``.

        If the given user may log in, this method should return None.
        """
        if not user.is_active:
            raise forms.ValidationError(
                self.error_messages['inactive'],
                code='inactive',
            )

    def get_user_id(self):
        if self.user_cache:
            return self.user_cache.id
        return None

    def get_user(self):
        return self.user_cache



# -*- coding: utf-8 -*-


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

import uuid
from django_enumfield import enum

from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.contrib.filefields.models import FileField


User = get_user_model()


# Create your models here.
class TicketPriority(enum.Enum):
    LOW = 0
    MEDIUM = 1
    HIGH = 2

    labels = {
        LOW: _('Low'),
        MEDIUM: _('Medium'),
        HIGH: _('High')
    }


class TicketStatus(enum.Enum):
    OPEN = 0
    WAITING_FOR_REVIEW = 1
    REVIEWED = 2
    CLOSE = 3

    labels = {
        OPEN: _('Open'),
        WAITING_FOR_REVIEW: _('Waiting for review'),
        REVIEWED: _('Reviewed'),
        CLOSE: _('Close')
    }


class TicketDepartment(enum.Enum):
    TECHNICAL = 0
    SALE = 1

    labels = {
        TECHNICAL: _('Technical'),
        SALE: _('Sale'),
    }


class TicketManager(BaseModelManager):

    def get_queryset(self):
        return super(TicketManager, self).get_queryset()

    def active(self):
        return super(TicketManager, self).active().filter(is_active=True)


class Ticket(BaseModel):
    user_obj = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))
    title = models.CharField(max_length=255, verbose_name=_('Title'))
    priority = enum.EnumField(TicketPriority, default=TicketPriority.LOW, verbose_name=_('Priority'))
    status = enum.EnumField(TicketStatus, default=TicketStatus.OPEN, verbose_name=_('Status'))
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, verbose_name=_('UUID'))
    department = enum.EnumField(TicketDepartment, default=TicketDepartment.TECHNICAL, verbose_name=_('Department'))

    is_active = models.BooleanField(default=True, verbose_name=_('Is Active'))

    objects = TicketManager()

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = _('Ticket')
        verbose_name_plural = _('Tickets')


class TicketConversationManager(models.Manager):

    def get_queryset(self):
        return super(TicketConversationManager, self).get_queryset()

    def active(self):
        return super(TicketConversationManager, self).get_queryset().filter(is_active=True)


class TicketConversation(models.Model):
    user_obj = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))
    ticket_obj = models.ForeignKey(Ticket, on_delete=models.CASCADE, verbose_name=_('Ticket'))
    content = models.TextField(verbose_name=_('Content'))
    files_obj = models.ManyToManyField(FileField, verbose_name=_('Files'))

    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = TicketConversationManager()

    def __str__(self):
        return "%s" % self.id

    class Meta:
        ordering = ['-created_at']
        verbose_name = _('Ticket Conversation')
        verbose_name_plural = _('Ticket Conversations')



from django.conf.urls import url, include

from .views import ReviewListAPIView, ReviewDetailAPIView, ReviewCreateAPIView, BaseReviewStatusSetAPIView

app_name = 'reviews'

urlpatterns = [
    url(r'^$', ReviewListAPIView.as_view(), name='list'),
    url(r'^(?P<review_id>\d+)/status-set/$', BaseReviewStatusSetAPIView.as_view(), name='status-set'),
    url(r'^(?P<review_id>\d+)/$', ReviewDetailAPIView.as_view(), name='details'),
    url(r'^model/(?P<model_id>\d+)/$', ReviewListAPIView.as_view(), name='models-list'),
    url(r'^(?P<review_id>\d+)/like/', include('aparnik.contrib.reviews.api.urls-like', namespace='like')),
    url(r'^(?P<review_id>\d+)/dislike/', include('aparnik.contrib.reviews.api.urls-dislike', namespace='dislike')),
    url(r'^create/$', ReviewCreateAPIView.as_view(), name='create'),
]


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-11 12:36


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0009_auto_20190511_1212'),
    ]

    operations = [
        migrations.AddField(
            model_name='filefield',
            name='is_decrypt_needed',
            field=models.BooleanField(default=False, verbose_name='Is decrypt needed?'),
        ),
    ]



from django.utils.text import Truncator
from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer

import datetime

from aparnik.api.serializers import ModelSerializer
from aparnik.utils.utils import is_app_installed
from aparnik.settings import Setting
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
from aparnik.contrib.categories.api.serializers import CategoryDetailsPolymorphicSerializer
from aparnik.packages.shops.products.api.serializers import ProductDetailSerializer, ProductListSerializer
from aparnik.packages.shops.files.api.serializers import FileListSerializer, FileDetailSerializer
from aparnik.packages.educations.progresses.api.serializers import ProgressListSerializer
from ..models import Course, CourseFile, BaseCourse, CourseSummary


# Total TIme Details Serializer
class CourseSummaryDetailSerializer(ModelSerializer):

    def __init__(self, *args, **kwargs):
        super(CourseSummaryDetailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = CourseSummary
        fields = [
            'total_time',
            'file_count',
            'file_count_preview',
            'type',
        ]


# Base Course Details Serializer
class BaseCourseListSerializer(ProductListSerializer):
    summary = serializers.SerializerMethodField()
    short_description = serializers.SerializerMethodField()

    class Meta:
        model = BaseCourse
        fields = ProductListSerializer.Meta.fields + [
            'summary',
            'short_description',
        ]
        read_only_fields = ProductListSerializer.Meta.read_only_fields + ['summary',
                                                                          'short_description'
                                                                          ]

    def get_summary(self, obj):
        return CourseSummaryDetailSerializer(obj.total_times.all(), many=True, read_only=True,
                                             context=self.context).data

    def get_short_description(self, obj):
        return Truncator(obj.description).words(30)


# Base Course Details Serializer
class BaseCourseDetailsSerializer(BaseCourseListSerializer, ProductDetailSerializer):
    class Meta:
        model = BaseCourse
        fields = BaseCourseListSerializer.Meta.fields + ProductDetailSerializer.Meta.fields + [
            'description',
        ]
        read_only_fields = ProductDetailSerializer.Meta.read_only_fields + BaseCourseListSerializer.Meta.read_only_fields + []


# Course List Serializer
class CourseListSerializer(BaseCourseListSerializer):
    cover = serializers.SerializerMethodField()
    banner = serializers.SerializerMethodField()
    child_count = serializers.SerializerMethodField()
    url_file = serializers.SerializerMethodField()
    url_file_create = serializers.SerializerMethodField()
    url_child = serializers.SerializerMethodField()
    dependency_title = serializers.SerializerMethodField()
    teachers = serializers.SerializerMethodField()
    progress_percentage = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(CourseListSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
        # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context']

    class Meta:
        model = Course
        fields = BaseCourseListSerializer.Meta.fields + [
            'cover',
            'banner',
            'child_count',
            'sort',
            'is_free',
            'url_child',
            'depth',
            'url_file',
            'url_file_create',
            'dependency_title',
            'publish_month',
            'publish_week',
            'publish_day',
            'teachers',
            'progress_percentage',
        ]
        read_only_fields = BaseCourseListSerializer.Meta.read_only_fields + ['cover', 'banner',
                                                                             'child_count', 'url_child',
                                                                             'url_file', 'url_file_create',
                                                                             'dependency_title', 'teachers',
                                                                             'progress_percentage'
                                                                             ]

    def get_cover(self, obj):
        return FileFieldListSerailizer(obj.cover, many=False, read_only=True,
                                       context=self.context).data if obj.cover else None

    def get_banner(self, obj):
        return FileFieldListSerailizer(obj.banner, many=False, read_only=True,
                                       context=self.context).data if obj.banner else None

    def get_child_count(self, obj):
        return Course.objects.childs_count(course_obj=obj.id)

    def get_url_child(self, obj):
        if Course.objects.childs_count(obj) > 0:
            return self.context['request'].build_absolute_uri(obj.get_api_child_uri())
        return None

    def get_url_file(self, obj):
        if obj.coursefile_set.count() > 0:
            return self.context['request'].build_absolute_uri(obj.get_api_file_uri())
        return None

    def get_url_file_create(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_file_create_uri())

    # def get_file_count(self, obj):
    #     return obj.get_files_count()

    def get_dependency_title(self, obj):
        return obj.dependency_obj.title if obj.dependency_obj is not None else None

    def get_teachers(self, obj):
        from aparnik.packages.educations.teachers.api.serializers import TeacherListPolymorphicSerializer
        return TeacherListPolymorphicSerializer(obj.teachers.active(), many=True, read_only=True,
                                                context=self.context).data

    def get_progress_percentage(self, obj):
        user = self.context['request'].user
        if user.is_authenticated:
            progress_summary = obj.progress_summaries.this_user(user).last()
            if progress_summary:
                return progress_summary.percentage
        return 0.0


# Course Details Serializer
class CourseDetailSerializer(CourseListSerializer, BaseCourseDetailsSerializer):
    parent = serializers.SerializerMethodField()
    category = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(CourseDetailSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
        # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = Course
        fields = CourseListSerializer.Meta.fields + BaseCourseDetailsSerializer.Meta.fields + [
            'category',
            'parent',
        ]
        read_only_fields = CourseListSerializer.Meta.read_only_fields + BaseCourseDetailsSerializer.Meta.read_only_fields + [
            'category',
            'parent']

    def get_category(self, obj):
        return CategoryDetailsPolymorphicSerializer(obj.category_obj, many=False, read_only=True,
                                                    context=self.context).data

    def get_parent(self, obj):
        return BaseCourseListPolymorphicSerializer(obj.parent_obj, many=False, read_only=True,
                                                   context=self.context).data if obj.parent_obj is not None else None

    def get_share_string(self, obj):
        url = Setting.objects.get(key='APP_LANDING_PAGE_URL').get_value()
        return '%s \n %s \n %s' % (url, obj.title, self.get_short_description(obj))


class CourseCreateSerializer(CourseDetailSerializer, BaseCourseDetailsSerializer):
    parent_obj_id = serializers.IntegerField(required=False, allow_null=True, write_only=True)
    banner_id = serializers.IntegerField(required=True, write_only=True)
    cover_id = serializers.IntegerField(required=True, write_only=True)

    def __init__(self, *args, **kwargs):
        super(CourseCreateSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Course
        fields = CourseDetailSerializer.Meta.fields + BaseCourseDetailsSerializer.Meta.fields + [
            'parent_obj_id',
            'banner_id',
            'cover_id',
        ]
        read_only_fields = CourseDetailSerializer.Meta.read_only_fields + BaseCourseDetailsSerializer.Meta.read_only_fields + [
        ]


# File List Serializer
class CourseFileListSerializer(FileListSerializer):
    url_progress_set = serializers.SerializerMethodField()
    progress_status = serializers.SerializerMethodField()
    time = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(CourseFileListSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
        # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = CourseFile
        fields = FileListSerializer.Meta.fields + [
            'sort',
            'time',
            'url_progress_set',
            'progress_status',
        ]
        read_only_fields = FileListSerializer.Meta.read_only_fields + ['url_progress_set',
                                                                       'progress_status', 'time']

    def get_url_progress_set(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_progress_set_uri())

    def get_progress_status(self, obj):
        user = self.context['request'].user
        progress = obj.progresses_set.this_user(user)
        if progress.count() > 0:
            return ProgressListSerializer(progress.last(), many=False, read_only=True, context=self.context).data
        return None

    def get_time(self, obj):
        return str(datetime.timedelta(seconds=obj.seconds))


# File Details Serializer
class CourseFileDetailSerializer(CourseFileListSerializer, FileDetailSerializer):
    course = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(CourseFileDetailSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
        # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = CourseFile
        fields = CourseFileListSerializer.Meta.fields + FileDetailSerializer.Meta.fields + [
            'course',
        ]
        read_only_fields = CourseFileListSerializer.Meta.read_only_fields + FileDetailSerializer.Meta.read_only_fields + [
            'course']

    def get_course(self, obj):
        return BaseCourseListPolymorphicSerializer(obj.course_obj, many=False, read_only=True,
                                                   context=self.context).data


class CourseFileCreateSerializer(CourseFileDetailSerializer, FileDetailSerializer):
    file_obj_id = serializers.IntegerField(required=True, write_only=True)

    def __init__(self, *args, **kwargs):
        super(CourseFileCreateSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = CourseFile
        fields = CourseFileDetailSerializer.Meta.fields + FileDetailSerializer.Meta.fields + [
            'password',
            'iv',
            'file_obj_id'
        ]
        extra_kwargs = {'course_obj_id': {'write_only': True}}
        read_only_fields = CourseFileDetailSerializer.Meta.read_only_fields + FileDetailSerializer.Meta.read_only_fields + \
                           []


# Base Course List Polymorphic
class BaseCourseListPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            BaseCourse: BaseCourseListSerializer,
            Course: CourseListSerializer,
        }
        super(BaseCourseListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


# Base Course Details Polymorphic
class BaseCourseDetailsPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            BaseCourse: BaseCourseDetailsSerializer,
            Course: CourseDetailSerializer,
        }
        super(BaseCourseDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.api.serializers import BaseSegmentListSerializer, BaseSegmentDetailSerializer
    from ..models import CourseSegment


    # Base Segment ListSegmentReview
    class SegmentCourseListSerializer(BaseSegmentListSerializer):
        class Meta:
            model = CourseSegment
            fields = BaseSegmentListSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentListSerializer.Meta.read_only_fields + []


    # Book Segment Details
    class SegmentCourseDetailSerializer(BaseSegmentDetailSerializer, SegmentCourseListSerializer):
        class Meta:
            model = CourseSegment
            fields = SegmentCourseListSerializer.Meta.fields + BaseSegmentDetailSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentDetailSerializer.Meta.read_only_fields + SegmentCourseListSerializer.Meta.read_only_fields + []



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.contrib.users.admin import get_update_at, get_user_search_fields
from .models import Bookmark


# Register your models here.
class BookmarkAdmin(BaseModelAdmin):
    list_display = ['user_obj', get_update_at]
    search_fields = get_user_search_fields('user_obj') + []
    raw_id_fields = ['user_obj',]
    list_filter = ['created_at']
    fields = []
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = BookmarkAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Bookmark

    def get_form(self, request, obj=None, **kwargs):
        form = super(BookmarkAdmin, self).get_form(request, obj, **kwargs)
        if is_app_installed('aparnik.packages.shops.products'):
            from aparnik.packages.shops.products.models import Product
            form.base_fields['model_obj'].queryset = Product.objects.active()
        return form


admin.site.register(Bookmark, BookmarkAdmin)



from .servers.share_servers import *
# from base import *
# #
DEBUG = True
#
# # INSTALLED_APPS += (
# #     #third party apps
# #         's3direct',
# #     #my apps
# # )
#
# #
MEDIA_URL = "/media/"
MEDIA_ROOT = os.path.join(os.path.dirname(BASE_DIR), "media")

STATIC_ROOT = os.path.join(BASE_DIR, 'staticfiles')
# STATICFILES_DIRS = [
#     os.path.join(BASE_DIR, "static"),
# ]
STATIC_URL = '/static/'
#
# USE_TZ = True




from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.api.serializers import ModelSerializer
from aparnik.utils.utils import is_app_installed
from ..models import BaseSegment, SegmentSort, PageSort


# Base Segment ListSegmentReview
class SegmentSortListSerializer(ModelSerializer):

    model = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(SegmentSortListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = SegmentSort
        fields = [
            'id',
            'model',
            'sort',
            'resourcetype'
        ]
        read_only_fields = ['id', 'model', 'sort']

    def get_model(self, obj):
        from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
        return ModelListPolymorphicSerializer(obj.model_obj.get_real_instance(), many=False, read_only=True, context=self.context).data

    def get_resourcetype(self, obj):
        return 'SegmentSort'


class PageSortListSerializer(ModelSerializer):

    model = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(PageSortListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = PageSort
        fields = [
            'id',
            'model',
            'sort',
            'resourcetype'
        ]
        read_only_fields = ['id', 'model', 'sort']

    def get_model(self, obj):
        from aparnik.contrib.segments.api.serializers import BaseSegmentListPolymorphicSerializer
        return BaseSegmentListPolymorphicSerializer(obj.segment_obj.get_real_instance(), many=False, read_only=True, context=self.context).data

    def get_resourcetype(self, obj):
        return 'PageSort'


class BaseSegmentListSerializer(ModelSerializer):
    url_list = serializers.SerializerMethodField()
    models = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(BaseSegmentListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BaseSegment
        fields = [
            'id',
            'title',
            'models',
            'url_list',
        ]
        read_only_fields = ['id']

    def get_models(self, obj):
        return SegmentSortListSerializer(obj.segment_sort_segment.all()[:10], many=True, read_only=True, context=self.context).data

    def get_url_list(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_list_model_uri())


    # Base Segment Details
class BaseSegmentDetailSerializer(BaseSegmentListSerializer):

    def __init__(self, *args, **kwargs):
        super(BaseSegmentDetailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BaseSegment
        fields = BaseSegmentListSerializer.Meta.fields + [

        ]
        read_only_fields = BaseSegmentListSerializer.Meta.read_only_fields + []


# Base Segment List Polymorphic
class BaseSegmentListPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):

        model_serializer_mapping = {
            BaseSegment: BaseSegmentListSerializer
        }

        # Segment Button
        if is_app_installed('aparnik.contrib.buttons'):
            from aparnik.contrib.buttons.models import ButtonSegment
            from aparnik.contrib.buttons.api.serializers import SegmentButtonListSerializer
            model_serializer_mapping[ButtonSegment] = SegmentButtonListSerializer

        # Segment Category
        if is_app_installed('aparnik.contrib.categories'):
            from aparnik.contrib.categories.models import CategorySegment
            from aparnik.contrib.categories.api.serializers import SegmentCategoryListSerializer
            model_serializer_mapping[CategorySegment] = SegmentCategoryListSerializer

        # Segment Slider
        if is_app_installed('aparnik.contrib.sliders'):
            from aparnik.contrib.sliders.models import Slider
            from aparnik.contrib.sliders.api.serializers import SegmentSliderListSerializer
            model_serializer_mapping[Slider] = SegmentSliderListSerializer

        # Segment Review
        if is_app_installed('aparnik.contrib.reviews'):
            from aparnik.contrib.reviews.models import BaseReviewSegment
            from aparnik.contrib.reviews.api.serializers import SegmentBaseReviewListSerializer
            model_serializer_mapping[BaseReviewSegment] = SegmentBaseReviewListSerializer

        # Segment Social network
        if is_app_installed('aparnik.contrib.socials'):
            from aparnik.contrib.socials.models import SocialNetworkSegment
            from aparnik.contrib.socials.api.serializers import SegmentSocialNetworkListSerializer
            model_serializer_mapping[SocialNetworkSegment] = SegmentSocialNetworkListSerializer

        # Segment Course
        if is_app_installed('aparnik.packages.educations.courses'):
            from aparnik.packages.educations.courses.models import CourseSegment
            from aparnik.packages.educations.courses.api.serializers import SegmentCourseListSerializer
            model_serializer_mapping[CourseSegment] = SegmentCourseListSerializer

        # Segment books
        if is_app_installed('aparnik.packages.educations.books'):
            from aparnik.packages.educations.books.models import BookSegment
            from aparnik.packages.educations.books.api.serializers import SegmentBookListSerializer
            model_serializer_mapping[BookSegment] = SegmentBookListSerializer

        self.model_serializer_mapping = model_serializer_mapping
        super(BaseSegmentListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


# Base Segment Details Polymorphic
class BaseSegmentDetailsPolymorphicSerializer(PolymorphicSerializer):

    def __init__(self, *args, **kwargs):
        model_serializer_mapping = {
            BaseSegment: BaseSegmentDetailSerializer
        }

        # Segment Button
        if is_app_installed('aparnik.contrib.buttons'):
            from aparnik.contrib.buttons.models import ButtonSegment
            from aparnik.contrib.buttons.api.serializers import SegmentButtonDetailSerializer
            model_serializer_mapping[ButtonSegment] = SegmentButtonDetailSerializer

        # Segment Category
        if is_app_installed('aparnik.contrib.categories'):
            from aparnik.contrib.categories.models import CategorySegment
            from aparnik.contrib.categories.api.serializers import SegmentCategoryDetailSerializer
            model_serializer_mapping[CategorySegment] = SegmentCategoryDetailSerializer

        # Segment Slider
        if is_app_installed('aparnik.contrib.sliders'):
            from aparnik.contrib.sliders.models import Slider
            from aparnik.contrib.sliders.api.serializers import SegmentSliderDetailSerializer
            model_serializer_mapping[Slider] = SegmentSliderDetailSerializer

        # Segment Review
        if is_app_installed('aparnik.contrib.reviews'):
            from aparnik.contrib.reviews.models import BaseReviewSegment
            from aparnik.contrib.reviews.api.serializers import SegmentBaseReviewDetailSerializer
            model_serializer_mapping[BaseReviewSegment] = SegmentBaseReviewDetailSerializer

        # Segment Social network
        if is_app_installed('aparnik.contrib.socials'):
            from aparnik.contrib.socials.models import SocialNetworkSegment
            from aparnik.contrib.socials.api.serializers import SegmentSocialNetworkDetailSerializer
            model_serializer_mapping[SocialNetworkSegment] = SegmentSocialNetworkDetailSerializer

        # Segment Course
        if is_app_installed('aparnik.packages.educations.courses'):
            from aparnik.packages.educations.courses.models import CourseSegment
            from aparnik.packages.educations.courses.api.serializers import SegmentCourseDetailSerializer
            model_serializer_mapping[CourseSegment] = SegmentCourseDetailSerializer

        # Segment books
        if is_app_installed('aparnik.packages.educations.books'):
            from aparnik.packages.educations.books.models import BookSegment
            from aparnik.packages.educations.books.api.serializers import SegmentBookDetailSerializer
            model_serializer_mapping[BookSegment] = SegmentBookDetailSerializer

        self.model_serializer_mapping = model_serializer_mapping
        super(BaseSegmentDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-21 18:56


from django.db import migrations
import uuid


def order_generate_uuid(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Object = apps.get_model('orders', 'Order')
    for obj in Object.objects.all():
        uu = uuid.uuid4()
        while Object.objects.filter(uuid=uu).count() > 0:
            uu = uuid.uuid4()
        obj.uuid = uu
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0006_order_uuid'),
    ]

    operations = [
        migrations.RunPython(order_generate_uuid, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-26 12:10


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0024_devicelogin_device_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='devicelogin',
            name='device_type_temp',
            field=models.CharField(blank=True, choices=[('a', 'Android'), ('i', 'iOS')], max_length=1, null=True, verbose_name='Device Type'),
        ),
    ]



# Generated by Django 2.2 on 2019-06-15 14:30

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0005_notificationsystem'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='from_user_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notificaiton_from_user', to=settings.AUTH_USER_MODEL, verbose_name='از کاربر'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='type',
            field=models.CharField(choices=[('i', 'اطلاعات'), ('s', 'موفقیت'), ('w', 'هشدار'), ('e', 'خطا')], max_length=1, verbose_name='Type'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-05 11:07


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0009_auto_20181105_1424'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='basecourse',
            options={'verbose_name': '\\u062f\\u0648\\u0631\\u0647 \\u067e\\u0627\\u06cc\\u0647', 'verbose_name_plural': '\\u062f\\u0648\\u0631\\u0647 \\u0647\\u0627\\u06cc \\u067e\\u0627\\u06cc\\u0647'},
        ),
        migrations.AlterModelOptions(
            name='course',
            options={'verbose_name': '\\u062f\\u0648\\u0631\\u0647', 'verbose_name_plural': '\\u062f\\u0648\\u0631\\u0647 \\u0647\\u0627'},
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.contrib.users.admin import get_user_search_fields
from .models import Payment


# Register your models here.
class PaymentAdmin(BaseModelAdmin):
    list_display = ['bank_reference', 'uuid', 'user', 'method', 'status', 'order_obj', 'call_back_url', 'created_at', 'update_at']
    list_filter = ['status', 'created_at']
    search_fields = get_user_search_fields('user') + ['uuid']
    # change_list_template = 'admin/change_list_payment.html'
    raw_id_fields = ['order_obj']
    fields = []
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = PaymentAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        Payment

    def changelist_view(self, request, extra_context=None):
        response = super(PaymentAdmin, self).changelist_view(request, extra_context=extra_context, )
        try:
            qs = response.context_data['cl'].queryset
            qs1 = Payment.objects.values('created_at', 'status')
            # qs2 = payment.objects.values('id', 'coupon__value', 'coupon__type', )

        except (AttributeError, KeyError):
            return response
        response.context_data["date_status"] = qs1

        return response

    def change_view(self, request, object_id, form_url='', extra_context=None):
        ''' customize edit form '''
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = False
        extra_context['show_save'] = False
        extra_context['show_save_and_add_another'] = False # this not works if has_add_permision is True
        return super(PaymentAdmin, self).change_view(request, object_id, extra_context=extra_context)


admin.site.register(Payment, PaymentAdmin)



# -*- coding: utf-8 -*-


from django.contrib import admin

from jalali_date import datetime2jalali

from aparnik.settings import aparnik_settings
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelStackedInline
from aparnik.contrib.users.admin import get_user_search_fields
from .models import BaseEducation, Education, Degree, FieldSubject, Institute, CityInstitute


# Register your models here.
class BaseEducationInline(BaseModelStackedInline):
    model = BaseEducation
    extra = 1
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = BaseEducationInline
        Klass_parent = BaseModelStackedInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude


    def get_queryset(self, request):
        return BaseEducation.objects.all()


#
class CityInline(BaseModelStackedInline):
    model = CityInstitute
    fk_name = 'Institute'
    extra = 1
    max_num = 1
    exclude = []
    raw_id_fields = ['city']
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = CityInline
        Klass_parent = BaseModelStackedInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude


def get_receive_date(obj):
    if obj.receive_date:
        return datetime2jalali(obj.receive_date).strftime(aparnik_settings.JALALI_FORMAT)
    return obj.receive_date


class EducationAdmin(BaseModelAdmin):
    fields = ['user_obj', 'degree_obj', 'institute_obj', 'field_subject_obj']
    list_display = ['user_obj', 'degree_obj', get_receive_date]
    list_filter = []
    search_fields = get_user_search_fields('user_obj')
    exclude = []
    dynamic_raw_id_fields = []
    raw_id_fields = ['user_obj', 'degree_obj', 'institute_obj', 'field_subject_obj']

    def __init__(self, *args, **kwargs):
        Klass = EducationAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Education


class DegreeAdmin(BaseModelAdmin):
    fields = []
    list_display = ['name']
    list_filter = []
    search_fields = []
    exclude = []
    raw_id_fields = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = DegreeAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Degree


class FieldSubjectAdmin(BaseModelAdmin):
    fields = []
    list_display = ['name']
    list_filter = []
    search_fields = []
    exclude = []
    raw_id_fields = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = FieldSubjectAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = FieldSubject


class InstituteAdmin(BaseModelAdmin):
    fields = []
    list_display = ['name']
    list_filter = []
    search_fields = []
    exclude = []
    dynamic_raw_id_fields = ['cities']
    raw_id_fields = []
    inlines = [
        CityInline,
    ]

    def __init__(self, *args, **kwargs):
        Klass = InstituteAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Institute


admin.site.register(Education, EducationAdmin)
admin.site.register(Degree, DegreeAdmin)
admin.site.register(FieldSubject, FieldSubjectAdmin)
admin.site.register(Institute, InstituteAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-03-19 17:45


from django.db import migrations
from faker import Faker


def username_mention(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    faker = Faker()

    User = apps.get_model('aparnik_users', 'User')
    for user in User.objects.all():
        username = faker.user_name()
        while User.objects.filter(username_mention=username).exists():
            username = faker.user_name()

        user.username_mention = username
        user.save()


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0017_auto_20190319_1744'),
    ]

    operations = [
        migrations.RunPython(username_mention, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.users.admin import get_update_at
from .models import ShortBlog


# Register your models here.
class ShortBlogAdmin(admin.ModelAdmin):
    fields = ['title', 'content']
    list_display = ['title', get_update_at]
    search_fields = ['title', 'content']
    readonly_fields = ['created_at', 'update_at']
    list_filter = ['created_at']
    # raw_id_fields = ['category', ]

    class Meta:
        ShortBlog

    def change_view(self, request, object_id, form_url='', extra_context=None):
        ''' customize edit form '''
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = False
        extra_context['show_save'] = False
        extra_context['show_save_and_add_another'] = False  # this not works if has_add_permision is True
        return super(ShortBlogAdmin, self).change_view(request, object_id, extra_context=extra_context)


admin.site.register(ShortBlog, ShortBlogAdmin)




# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-06-13 18:30


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('questionanswers', '0006_auto_20190613_1829'),
        ('reviews', '0007_auto_20190613_1829'),
    ]

    operations = [
        migrations.RenameField(
            model_name='qa',
            old_name='model_obj2',
            new_name='model_obj',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-04 15:05


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('progresses', '0003_auto_20181103_2233'),
    ]

    operations = [
        migrations.AlterField(
            model_name='progresssummary',
            name='percentage',
            field=models.FloatField(verbose_name='Percentage'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta

from aparnik.contrib.reviews.models import Review, ReviewSummary
from aparnik.contrib.basemodels.models import BaseModel
import datetime

User = get_user_model()


class Command(BaseCommand):
    # Show this when the user types help
    help = "calculate for Review Model"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        #
        reindex = False
        if Review.objects.update_needed().count() > 0:
            reindex = True

        if reindex:
            self.reindex()

        # Course.objects.update()
        Review.objects.update_needed().update(update_needed=False)
        finished_time = now()

        print(('reindex reviews %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))

    def reindex(self):
        models = []
        for obj in Review.objects.active():
            models.append(obj.model_obj)

        models = list(set(models))
        for obj in models:
            obj.review_average = Review.objects.model_review_avg(obj)
            obj.save()
            count_all = Review.objects.filter(model_obj=obj).count()

            for i in range(6):
                count = Review.objects.filter(rate=i, model_obj=obj).count()
                percentage = (float(count) / float(count_all)) * 100
                summary = None
                try:
                    summary = obj.review_summaries.get(rate=i)
                    summary.count = count
                    summary.percentage = percentage
                    summary.save()
                except:
                    summary = ReviewSummary.objects.create(rate=i, count=count, model=obj, percentage=percentage)



# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from rest_framework.renderers import JSONRenderer
from dateutil.relativedelta import relativedelta

from fcm_django.models import FCMDevice
from aparnik.utils.utils import get_request
from aparnik.contrib.settings.models import Setting
from aparnik.contrib.notifications.models import Notification
from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
from aparnik.settings import aparnik_settings


json = JSONRenderer()
User = get_user_model()
UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


class Command(BaseCommand):
    # Show this when the user types help
    help = "send notification"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        if Notification.objects.update_needed().count() > 0:
            self.reindex()

        finished_time = now()

        print(('send notifications %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))

    def reindex(self):
        icon = 'https://cdn.aparnik.com/static/website/img/logo-persian.png'
        request = get_request()
        try:
            setting = Setting.objects.get(key='LOGO_PROJECT_ICON')
            icon = setting.get_value()
        except:
            pass
        for obj in Notification.objects.update_needed():

            model_serial = ModelListPolymorphicSerializer(obj.model_obj,
                                                          many=False,
                                                          read_only=True,
                                                          context={'request': request},
                                                          ).data if obj.model_obj else None

            user_serial = UserSummeryListSerializer(obj.from_user_obj,
                                                    many=False,
                                                    read_only=True,
                                                    context={'request': request},
                                                    ).data if obj.from_user_obj else None
            try:
                devices = FCMDevice.objects.filter(user__in=obj.users.all()).order_by('device_id').distinct()
                # TODO: handle response to log and etc...
                sent_result = devices.send_message(
                    title=obj.title,
                    body=obj.description,
                    icon=icon,
                    data={
                        'model': model_serial,
                        'from_user': user_serial,
                        'type': obj.type,
                        'title': obj.title,
                        'description': obj.description,

                    }
                )
                obj.sent_result = sent_result
                obj.description_for_admin = None
                obj.update_needed = False
                obj.save()
            except Exception as inst:
                obj.description_for_admin = 'type: {0}\nargs: {1}\ninstance: {2}'.format(type(inst), inst.args, inst)
                obj.save()



# -*- coding: utf-8 -*-


from django.shortcuts import render
from django.urls import reverse
from django.http import HttpResponse
from django.shortcuts import redirect, get_object_or_404
from django.http import Http404
from zeep import Client

from aparnik.packages.shops.payments.models import Payment
from aparnik.settings import aparnik_settings, Setting
from .models import Bank


def send_request(request, payment):
    client = Client('https://www.zarinpal.com/pg/services/WebGate/wsdl')
    order = payment.order_obj
    description = 'خرید %s' %(order.items.first().product_obj.title)
    call_back_url = request.build_absolute_uri(reverse('aparnik:shops:zarinpals:verify', args=[payment.uuid]))
    merchent_code = Setting.objects.get(key='ZARINPAL_MERCHENT_CODE').get_value()
    result = client.service.PaymentRequest(
        merchent_code,
        order.get_total_cost(),
        description,
        '',
        '',
        call_back_url
    )
    if result.Status == 100:
        bank = Bank.objects.create(authority_id=result.Authority,
                                   status=result.Status,
                                   payment=payment)
        return redirect('https://www.zarinpal.com/pg/StartPay/%s/ZarinGate' %(str(result.Authority)))
    else:
        raise Http404 #HttpResponse('Error code: ' + str(result.Status))


def verify(request, uuid):
    client = Client('https://www.zarinpal.com/pg/services/WebGate/wsdl')
    payment = get_object_or_404(Payment, uuid=uuid)

    if payment.status != Payment.STATUS_WAITING:
        raise Http404

    merchent_code = Setting.objects.get(key='ZARINPAL_MERCHENT_CODE').get_value()
    result = client.service.PaymentVerification(merchent_code, request.GET['Authority'], payment.order_obj.get_total_cost())
    bank = get_object_or_404(Bank, authority_id=request.GET['Authority'])

    if bank.payment != payment:
        raise Http404

    bank.ref_id = result.RefID
    bank.status = result.Status
    bank.save()
    if result.Status == 100 or result.Status == 101:
        payment.success()
    else:
        # return HttpResponse('Transaction failed.\nStatus: ' + str(result.Status))
        payment.cancel()

    # response = HttpResponse("", status=302)
    # url = bank_obj.get_form_url()
    # response['Location'] = url
    # response['Location'] = payment.get_call_back_url()

    # return response
    from aparnik.packages.shops.payments.views import payment as payment_view
    return payment_view(request, uuid=payment.uuid)
    # return render(request=request, template_name='suit/payunsuc.html', context={'obj': payment}, status=400)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 17:45


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('questionanswers', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='qa',
            options={'verbose_name': '\\u067e\\u0631\\u0633\\u0634 \\u0648 \\u067e\\u0627\\u0633\\u062e', 'verbose_name_plural': '\\u067e\\u0631\\u0633\\u0634 \\u0648 \\u067e\\u0627\\u0633\\u062e \\u0647\\u0627'},
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.conf import settings
from jalali_date import datetime2jalali

from aparnik.contrib.users.admin import get_update_at
from aparnik.contrib.invitation.models import Invite

# Register your models here.
class InviteAdmin(admin.ModelAdmin):
    fields = ['id', 'invite', 'invited_by', 'update_at']
    list_display = ['id', 'invite', 'invited_by', get_update_at]
    search_fields = ['invite', 'invited_by']
    list_filter = ['update_at']
    readonly_fields = ['invite', 'invited_by', 'update_at']

    class Meta:
        Invite

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False

    def change_view(self, request, object_id, form_url='', extra_context=None):
        ''' customize edit form '''
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = False
        extra_context['show_save'] = False
        extra_context['show_save_and_add_another'] = False # this not works if has_add_permision is True
        return super(InviteAdmin, self).change_view(request, object_id, extra_context=extra_context)


admin.site.register(Invite, InviteAdmin)




from django.conf.urls import url, include

from .views import PublisherListAPIView, PublisherDetailsAPIView

app_name = 'books'

urlpatterns = [
    url(r'^$', PublisherListAPIView.as_view(), name='list'),
    url(r'^(?P<publisher_id>\d+)/$', PublisherDetailsAPIView.as_view(), name='details'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-04-28 16:35


import ckeditor_uploader.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0011_auto_20181214_1330'),
    ]

    operations = [
        migrations.AddField(
            model_name='basecourse',
            name='content',
            field=ckeditor_uploader.fields.RichTextUploadingField(blank=True, null=True, verbose_name='\\u0645\\u062d\\u062a\\u0648\\u0627'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-13 18:53


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('segments', '0010_basesegment_pages'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='basesegment',
            name='sort',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-13 18:48


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0005_auto_20181108_1132'),
    ]

    operations = [
        migrations.RenameField(
            model_name='product',
            old_name='is_free',
            new_name='is_free_field',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-26 10:43


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0018_remove_coursefile_time'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='coursesummary',
            options={'ordering': ('id',), 'verbose_name': '\\u062e\\u0644\\u0627\\u0635\\u0647 \\u062f\\u0648\\u0631\\u0647', 'verbose_name_plural': '\\u062e\\u0644\\u0627\\u0635\\u0647 \\u062f\\u0648\\u0631\\u0647'},
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models import Q, Count, Case, When, Value, BooleanField, Subquery, OuterRef
from django.core.validators import MaxValueValidator, MinValueValidator
from django.urls import reverse
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from dateutil.relativedelta import relativedelta
from decimal import Decimal

from aparnik.utils.formattings import formatprice
from aparnik.utils.utils import round
from aparnik.contrib.settings.models import Setting
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.contrib.filefields.models import FileField

from aparnik.utils.fields import PriceField
from aparnik.contrib.sliders.models import SliderSegment

import decimal


# Create your models here.
# Product Manager
class ProductManager(BaseModelManager):

    def get_queryset(self):
        return super(ProductManager, self).get_queryset()

    def get_this_user(self, user):
        if not user.is_authenticated:
            return Product.objects.none()
        return self.get_queryset().all()

    def active(self, user=None):
        from aparnik.packages.shops.orders.models import Order
        from aparnik.packages.shops.productssharing.models import ProductSharing
        # TODO: Count is not correct
        return super(ProductManager, self).active(user=user).filter(is_draft=False).annotate(
            # user_invited_count_with_query=Count('productsharing_product', filter=ProductSharing.query_share_with(user, prefix='productsharing_product')),
            # is_user_invited_with_query=Case(When(user_invited_count_with_query__gte=1, then=Value(True)), default=Value(False),
            #                        output_field=BooleanField()),
            # my_product_count=Subquery(
            #     Order.objects..filter(
            #         is_paid=True,
            #         event=OuterRef('pk')
            #     ).values('event')
            #         .annotate(cnt=Count('pk'))
            #         .values('cnt'),
            #     output_field=models.IntegerField()
            # ),
            # my_product_count=Count('orderitem_set__order_obj', filter=Order.query_success('orderitem_set__order_obj', user=user)),
            # is_buy_with_query=Case(When(my_product_count__gte=1, then=Value(True)), default=Value(False), output_field=BooleanField()),
            # has_permit_with_query=Case(When(Q(Q(is_buy_with_query=True) | Q(is_free_field=True) | Q(is_user_invited_with_query=True)), then=Value(True)), default=Value(False), output_field=BooleanField()),
        )

    def get_wallet(self):
        return self.get(id=Setting.objects.get(key='PRODUCT_WALLET_ID').get_value())


# Product Model
class Product(BaseModel):
    # DELIVERY_TYPE status:
    DELIVERY_TYPE_SHIPPABLE = 'sh'
    DELIVERY_TYPE_DOWNLOADABLE = 'dw'
    DELIVERY_TYPE_CHOICES = (
        (DELIVERY_TYPE_SHIPPABLE, _('Shippable')),
        (DELIVERY_TYPE_DOWNLOADABLE, _('Downloadable')),
    )
    CURRENCY_IRR = 'IRR'
    CURRENCY_DOLLAR = 'D'
    CURRENCY_CHOICES = (
        (CURRENCY_IRR, _('IRR')),
        (CURRENCY_DOLLAR, _('Dollar')),
    )
    title = models.CharField(max_length=255, verbose_name=_('Title'))
    currency = models.CharField(max_length=10, default=CURRENCY_IRR, choices=CURRENCY_CHOICES, verbose_name=_('Currency'))
    price_fabric = PriceField(verbose_name=_('Price Fabric'))
    slider_segment_obj = models.ForeignKey(SliderSegment, null=True, blank=True, related_name='products_sliders', on_delete=models.CASCADE, verbose_name=_('Slider Segment'))
    is_free_field = models.BooleanField(default=False, verbose_name=_('Is Free'))
    discount_percent_value = models.PositiveIntegerField(default=0, validators=[MaxValueValidator(100), MinValueValidator(0)], verbose_name=_('Discount Price'))
    discount_percent_expire = models.DateTimeField(null=True, blank=True, help_text=_('If blank, without expire'), verbose_name=_('Discount Expire'))
    is_discount_percent_expire_show = models.BooleanField(default=False, help_text=_('If True countdown is available'), verbose_name=_('Is Discount Expire Show'))
    is_tax = models.BooleanField(default=False, verbose_name=_('Is tax?'))
    delivery_type = models.CharField(max_length=10, blank=True, choices=DELIVERY_TYPE_CHOICES,
                                     verbose_name=_('Delivery type'))
    further_details = models.TextField(null=True, blank=True, verbose_name=_('Extra Description'))
    properties = models.ManyToManyField('ProductProperty', through='ProductPropertyMembership', verbose_name=_('Product Properties'))
    # این فیلد به درصد می باشد و در محاسبات تبدیل به تعداد بن خواهد شد
    aparnik_bon_return_value = models.IntegerField(default=-1,
                                                   validators=[MinValueValidator(-1), MaxValueValidator(100)],
                                                           help_text=_(
                                                               'If the number is set to -1, then the value in the settings will apply'),
                                                           verbose_name=_('Bon Return'))
    aparnik_bon_return_expire_value = models.IntegerField(default=-1,
                                                           help_text=_(
                                                               'If the number is set to -1, then the value in the settings will apply'),
                                                           verbose_name=_('Bon Return Expire ( Hours )'))
    # این فیلد به درصد می باشد و در محاسبات تبدیل به تعداد بن خواهد شد
    maximum_use_aparnik_bon_value = models.IntegerField(default=-1,
                                                        validators=[MinValueValidator(-2), MaxValueValidator(100)],
                                                        help_text=_(
                                                                      'If the number is set to -1, then the value in the settings will apply, If the number is set to -2, then no limitation execution.'),
                                                                  verbose_name=_('Maximum use aparnik bon'))
    has_permit_use_wallet_value = models.IntegerField(default=-1,
                                                validators=[MaxValueValidator(1), MinValueValidator(-1)],
                                                help_text=_('If the number is set to -1, then the value in the settings will apply'),
                                                verbose_name=_('Has Permit use wallet'))
    is_draft = models.BooleanField(default=False, verbose_name=_('Is draft'))

    objects = ProductManager()

    def __init__(self, *args, **kwargs):
        super(Product, self).__init__(*args, **kwargs)

    class Meta:
        verbose_name = _('Product')
        verbose_name_plural = _('Products')

    def __str__(self):
        return '%s %s' % (self.id, self.title)

    @property
    def price(self):
        irt = 1
        if self.currency == self.CURRENCY_DOLLAR:
            irt = Setting.objects.get(key='DOLLAR_TO_IRR').get_value() / 10
        price_fabric = self.price_fabric * Decimal(irt)
        price = round(price_fabric - (price_fabric * decimal.Decimal(self.discount_percent / 100.0)))
        return price

    @property
    def price_fabric_string(self):
        return '%s' % formatprice.format_price(self.price_fabric)

    @property
    def price_string(self):
        return '%s' % formatprice.format_price(self.price)

    @property
    def discount_percent(self):
        if self.discount_percent_expire:
            return self.discount_percent_value if self.discount_percent_expire > now() else 0

        else:
            return self.discount_percent_value

    @property
    def is_free(self):
        return self.is_free_field

    # آیا این محصول جزو محصولات سابسکریپشن هست یا خیر
    # اگر هست باید از لینک های سابسکرایب خریداری بشه.
    @property
    def is_subscription(self):
        return self.subscriptions.active().count() > 0

    def buy_success(self, order):
        # override if you want custom process when order success
        pass

    def get_api_products_sharing_uri(self):
        return reverse('aparnik-api:shops:productssharing:product-user-share-list', args=[self.id])

    def get_api_product_sharing_add_uri(self):
        return reverse('aparnik-api:shops:productssharing:product-share-set', args=[self.id])

    # آیا این کاربر کالا را خریداری کرده است؟
    def is_buy(self, user):
        if not user.is_authenticated:
            return False
        from aparnik.packages.shops.orders.models import Order
        order = Order.objects.get_order_product(user=user, product=self)
        if order:
            return True
        return False

    # آیا این کاربر به این کالا دعوت شده است؟
    def is_user_invited(self, user):
        if not user.is_authenticated:
            return False
        from aparnik.packages.shops.productssharing.models import ProductSharing
        return ProductSharing.objects.share_with_this_user(user=user, product_id=self.id).exists()

    # بررسی دسترسی
    def has_permit(self, user):
        if self.is_free or self.is_user_invited(user) or self.is_buy(user):
            return True
        return False

    def get_title(self):
        return self.title

    def price_string_description(self, user):

        if not self.has_permit(user):
            return self.price_string

        if self.is_free:
            try:
                return Setting.objects.get(key='PRICE_PRODUCT_FREE_DESCRIPTION').get_value()
            except Exception as e:
                return 'رایگان'
        elif self.is_buy(user):
            try:
                return Setting.objects.get(key='PRICE_PRODUCT_BUY_DESCRIPTION').get_value()
            except Exception as e:
                return 'پرداخت شده'
        elif self.is_user_invited(user):
            try:
                return Setting.objects.get(key='PRICE_PRODUCT_SHARING_DESCRIPTION').get_value()
            except Exception as e:
                return 'دعوت شده'

        return self.price_string

    @property
    def aparnik_bon_return(self):
        value = self.aparnik_bon_return_value
        if value == -1:
            value = Setting.objects.get(key='APARNIK_BON_RETURN_DEFAULT_VALUE').get_value()
        value = ((self.price * value) / Decimal('100')) / Setting.objects.get(key='APARNIK_BON_VALUE').get_value()
        return value

    @property
    def aparnik_bon_return_value_string(self):
        price = round(self.aparnik_bon_return * Setting.objects.get(key='APARNIK_BON_VALUE').get_value())
        return '%s' % formatprice.format_price(price)

    @property
    def aparnik_bon_return_expire(self):
        value = self.aparnik_bon_return_expire_value
        if value == -1:
            value = Setting.objects.get(key='APARNIK_BON_RETURN_DEFAULT_EXPIRE_VALUE').get_value()
        return value

    @property
    def aparnik_bon_return_expire_date(self):
        value = self.aparnik_bon_return_expire
        if value == 0:
            return now() + relativedelta(years=200)
        return now() + relativedelta(hours=value)

    @property
    def maximum_use_aparnik_bon(self):
        value = self.maximum_use_aparnik_bon_value
        if value == -1:
            value = Setting.objects.get(key='MAXIMUM_USE_APARNIK_BON').get_value()
        if value == -2:
            import sys
            value = sys.maxsize

        value = ((self.price * value) / Decimal('100')) / Setting.objects.get(key='APARNIK_BON_VALUE').get_value()
        return value

    @property
    def has_permit_use_wallet(self):
        value = self.has_permit_use_wallet_value
        if value == -1:
            value = Setting.objects.get(key='HAS_PERMIT_DEFAULT_USE_WALLET').get_value()
        return value == 1


class ProductPropertyManager(models.Manager):

    def get_queryset(self):
        return super(ProductPropertyManager, self).get_queryset()

    def active(self):
        return self.get_queryset()


# Product Model
class ProductProperty(models.Model):

    icon = models.ForeignKey(FileField, on_delete=models.CASCADE, verbose_name=_('Icon'))
    title = models.CharField(max_length=100, verbose_name=_('Title'))

    objects = ProductPropertyManager()

    def __str__(self):
        return '%s %s' % (self.id, self.title)

    class Meta:
        verbose_name = _('Product Property')
        verbose_name_plural = _('Product Properties')


class ProductPropertyMembership(models.Model):
    product = models.ForeignKey(Product, on_delete=models.CASCADE, verbose_name=_('Product'))
    property = models.ForeignKey(ProductProperty, on_delete=models.CASCADE, verbose_name=_('Property'))
    value = models.CharField(max_length=255, verbose_name=_('Content'))

    class Meta:
        verbose_name = _('Product Property Membership')
        verbose_name_plural = _('Product Property Membership')

    def __str__(self):
        return self.value



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-06-13 18:29


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questionanswers', '0005_auto_20190613_1815'),
        ('reviews', '0006_auto_20190613_1815'),
    ]

    operations = [
        migrations.AlterField(
            model_name='qa',
            name='model_obj2',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='question_answers_set', to='basemodels.BaseModel', verbose_name='\\u0686\\u0647 \\u0686\\u06cc\\u0632 \\u0631\\u0627 \\u0646\\u0645\\u0627\\u06cc\\u0634 \\u062f\\u0647\\u062f\\u061f'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-


from django import forms
from django.forms.models import BaseInlineFormSet
from django.contrib import admin
from django.apps import apps

from aparnik.contrib.users.admin import get_update_at
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelTabularInline
from .models import BaseSegment


class SegmentSortInline(BaseModelTabularInline):
    model = BaseSegment.model_obj.through
    fields = ('model_obj', 'sort')
    extra = 1
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = SegmentSortInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude

    # def get_field_queryset(self, db, db_field, request):
    #     if db_field == 'model_obj':
    #         return
    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        """enable ordering drop-down alphabetically"""
        if db_field.name == 'model_obj':
            try:
                sp = self.parent_model._meta.label.split('.')
                app_label = sp[0]
                class_name = sp[1].replace('Segment', '')
                Model = apps.get_model(app_label, class_name)
                kwargs['queryset'] = Model.objects.active()
            except Exception:
                pass
        return super(SegmentSortInline, self).formfield_for_foreignkey(db_field, request, **kwargs)


class PageSortInline(BaseModelTabularInline):
    model = BaseSegment.pages.through
    fields = ('page_obj', 'sort')
    raw_id_fields = ('page_obj',)
    extra = 1
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = PageSortInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude



class SegmentAdmin(BaseModelAdmin):
    # fields = ['title', 'model_obj', 'is_active', 'is_home', 'is_library', 'sort', ]
    fields = ['title', ]
    list_display = ['title', get_update_at, ]
    search_fields = ['id', 'model_obj__id', 'title', ]
    # list_filter = ['is_active', 'is_home', 'is_library', 'created_at', ]
    list_filter = ['is_active', 'created_at', ]
    exclude = []
    raw_id_fields = []
    dynamic_raw_id_fields = []

    inlines = (PageSortInline, SegmentSortInline)

    def __init__(self, *args, **kwargs):
        Klass = SegmentAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = BaseSegment

    # def get_formsets_with_inlines(self, request, obj=None):
    #     for inline in self.get_inline_instances(request, obj):
    #         if isinstance(inline, SegmentSortInline):
    #             from aparnik.packages.educations.courses.models import Course
    #             if hasattr(inline.form.base_fields, 'model_obj'):
    #                 inline.form.base_fields['model_obj'].queryset = Course.objects.all()
    #         yield inline.get_formset(request, obj), inline

    # def get_formsets_with_inlines(self, request, obj=None):
    #     for inline in self.get_inline_instances(request, obj):
    #         # hide MyInline in the add view
    #         formset = None
    #         if isinstance(inline, SegmentSortInline):
    #             try:
    #                 sp = type(self).Meta.model._meta.label.split('.')
    #                 app_label = sp[0]
    #                 class_name = sp[1].replace('Segment', '')
    #                 Model = apps.get_model(app_label, class_name)
    #                 formset = inline.get_formset(request, obj, model_querysey=Model.objects.active())
    #             except Exception:
    #                 pass
    #         if not formset:
    #             formset = inline.get_formset(request, obj)
    #         yield formset, inline


admin.site.register(BaseSegment, SegmentAdmin)
# admin.site.register(SegmentReview, SegmentReviewAdmin)
# admin.site.register(SegmentCourse, SegmentCourseAdmin)
# admin.site.register(SegmentLibrary, SegmentLibraryAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-02 15:28


import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0009_user_co_sale_percentage_value'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='co_sale_percentage_value',
            field=models.IntegerField(default=-1, help_text='If the number is set to -1, then the value in the settings will apply', validators=[django.core.validators.MaxValueValidator(100), django.core.validators.MinValueValidator(-1)], verbose_name='Co Sale Percentage'),
        ),
        migrations.AlterField(
            model_name='user',
            name='limit_device_login',
            field=models.IntegerField(default=-1, help_text='If the number is set to -1, then the value in the settings will apply', verbose_name='Limit device login'),
        ),
    ]



from django.shortcuts import get_object_or_404
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated, IsAdminUser)
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.views import APIView
from rest_framework import filters
from django_filters.rest_framework import DjangoFilterBackend


from aparnik.utils.utils import is_app_installed
from aparnik.packages.shops.files.api.serializers import FileListPolymorphicSerializer
from .serializers import BaseCourseListPolymorphicSerializer, BaseCourseDetailsPolymorphicSerializer, \
    CourseFileCreateSerializer, CourseCreateSerializer
from ..models import Course, CourseFile


def get_course(kwargs):
    return int(kwargs['course_id'])


def get_step(kwargs):
    return int(kwargs['step_id'])


class CourseListAPIView(ListAPIView):
    serializer_class = BaseCourseListPolymorphicSerializer
    permission_classes = [AllowAny]
    filter_backends = (DjangoFilterBackend, filters.SearchFilter)
    search_fields = ['description', 'title', ]
    filter_fields = ('publish_month', 'publish_week')

    def get_queryset(self):
        queryset = None
        user = self.request.user
        if any(name in self.request.query_params for name in ['search', 'publish_month', 'publish_week']):
            queryset = Course.objects.active(user)
        else:
            queryset = Course.objects.courses()

        if 'course_id' in self.request.parser_context['kwargs']:
            queryset = Course.objects.childs(course_obj=self.request.parser_context['kwargs']['course_id'])

        return queryset


class CourseUserListAPIView(ListAPIView):
    serializer_class = BaseCourseListPolymorphicSerializer
    permission_classes = [IsAuthenticated]
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id', 'publish_month', 'publish_week')

    def get_queryset(self):
        user = self.request.user
        queryset = Course.objects.active()
        if is_app_installed('aparnik.packages.shops.orders'):
            from aparnik.packages.shops.orders.models import Order
            queryset = Course.objects.filter(
                id__in=Order.objects.get_this_user_bought(user=user, custom_type=Course).values_list('items__product_obj', flat=True))
        return queryset


class CourseDetailAPIView(RetrieveAPIView):
    serializer_class = BaseCourseDetailsPolymorphicSerializer
    queryset = Course.objects.active()
    permission_classes = [AllowAny]
    lookup_url_kwarg = 'course_id'
    lookup_field = 'id'


class CourseCreateAPIView(CreateAPIView):
    serializer_class = CourseCreateSerializer
    permission_classes = [IsAdminUser]

    def perform_create(self, serializer):
        serializer.save()


# class StepOrderAPIView(APIView):
#     permission_classes = [IsAuthenticated]
#
#     def post(self, request, step_id, format=None, *args, **kwargs):
#         status = HTTP_400_BAD_REQUEST
#         user = self.request.user
#         id = step_id
#         content = {
#             'success': False,
#             'msg': '',
#             'data': {
#             },
#         }
#         order = None
#         step = None
#         try:
#             step = get_object_or_404(Step.objects.all(), id=id)
#         except Exception as e:
#             content["msg"] = e
#             content["success"] = False
#             content["data"] = {}
#             return Response(content, status=status)
#
#         try:
#             item = OrderItem.objects.get(product=step, order_obj__user=user, order_obj__status=Order.STATUS_COMPLETE)
#             order = item.order
#         except:
#             order = Order.objects.create(user=user)
#             item = order.add_item(product=step)
#             order.save()
#
#         content['data']['order'] = OrderSummarySerializer(order, many=False, read_only=True,
#                                                           context={'request': self.request}).data
#         content['success'] = True
#         status = HTTP_200_OK
#         return Response(content, status=status)

#
class CourseFileListAPIView(ListAPIView):
    # serializer_class = BaseCourseListPolymorphicSerializer
    serializer_class = FileListPolymorphicSerializer
    permission_classes = [AllowAny]

    def perform_create(self, serializer):
        serializer.save()

    # filter_backends = (filters.SearchFilter,)

    # def get_queryset(self, *args, **kwargs):
    #     return File.objects.filter(course_obj=self.kwargs['course_id']).order_by('sort')
    def get_queryset(self, *args, **kwargs):
        course = get_object_or_404(Course.objects.active(), id=self.request.parser_context['kwargs']['course_id'])
        return CourseFile.objects.active().filter(course_obj=course)


class CourseFileCreateAPIView(CreateAPIView):
    serializer_class = CourseFileCreateSerializer
    permission_classes = [IsAdminUser]

    def perform_create(self, serializer):
        serializer.save(course_obj_id=self.kwargs['course_id'])


class FileDownloadRequestAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, file_id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user
        content = {
            'success': False,
            'msg': '',
            'data': {

            },
        }

        try:
            file = get_object_or_404(CourseFile.objects.get_this_user(user=user), id=file_id)

            key = self.request.data.get('key', None)
            if not key:
                content['msg'] += _('key is required')

            if content['msg']:
                raise ValidationError({'__all__': [_(content['msg'])]})

            (content["data"]["key"], content['data']['iv'], content["data"]["url_download"]) = file.get_key_url(
                user=user, key=key)
            content["data"]["url_download"] = self.request.build_absolute_uri(content["data"]["url_download"])
            content["success"] = True
            status = HTTP_200_OK

        except Exception as e:

            content["msg"] = e
            content["success"] = False
            content["data"] = {}
            status = HTTP_400_BAD_REQUEST

        return Response(content, status=status)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 17:45


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('segments', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='basesegment',
            options={'verbose_name': '\\u0628\\u062e\\u0634 \\u067e\\u0627\\u06cc\\u0647', 'verbose_name_plural': '\\u0628\\u062e\\u0634 \\u067e\\u0627\\u06cc\\u0647'},
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='\\u0633\\u0627\\u062e\\u062a\\u0647 \\u0634\\u062f\\u0647 \\u062f\\u0631'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='\\u0641\\u0639\\u0627\\u0644\\u061f'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='model_obj',
            field=models.ManyToManyField(to='basemodels.BaseModel', verbose_name='\\u0686\\u0647 \\u0686\\u06cc\\u0632 \\u0631\\u0627 \\u0646\\u0645\\u0627\\u06cc\\u0634 \\u062f\\u0647\\u062f\\u061f'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='pages',
            field=models.ManyToManyField(blank=True, related_name='segment_pages', to='pages.Page', verbose_name='\\u0635\\u0641\\u062d\\u0647'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='sort',
            field=models.IntegerField(default=0, verbose_name='\\u0645\\u0631\\u062a\\u0628 \\u0633\\u0627\\u0632\\u06cc'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='title',
            field=models.CharField(max_length=60, verbose_name='\\u0639\\u0646\\u0648\\u0627\\u0646'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='update_at',
            field=models.DateTimeField(auto_now=True, verbose_name='\\u0628\\u0631\\u0648\\u0632\\u0631\\u0633\\u0627\\u0646\\u06cc \\u0634\\u062f\\u0647 \\u062f\\u0631'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('segments', '0001_initial'),
        ('basemodels', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('reviews', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaseReviewSegment',
            fields=[
                ('basesegment_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='segments.BaseSegment')),
            ],
            options={
                'verbose_name': 'Review Segment',
                'verbose_name_plural': 'Reviews Segments',
            },
            bases=('segments.basesegment',),
        ),
        migrations.CreateModel(
            name='Like',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_like', models.BooleanField(default=False, verbose_name='Is Like')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
            ],
            options={
                'verbose_name': 'Like',
                'verbose_name_plural': 'Likes',
            },
        ),
        migrations.CreateModel(
            name='Review',
            fields=[
                ('basereview_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='reviews.BaseReview')),
                ('rate', models.FloatField(verbose_name='Rate')),
            ],
            options={
                'verbose_name': 'Review',
                'verbose_name_plural': 'Reviews',
            },
            bases=('reviews.basereview',),
        ),
        migrations.AddField(
            model_name='like',
            name='review_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='reviews.BaseReview', verbose_name='Review'),
        ),
        migrations.AddField(
            model_name='like',
            name='user_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='basereview',
            name='model_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='review_obj', to='basemodels.BaseModel', verbose_name='Model'),
        ),
        migrations.AddField(
            model_name='basereview',
            name='parent_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='child', to='reviews.BaseReview', verbose_name='Parent'),
        ),
        migrations.AddField(
            model_name='basereview',
            name='user_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 16:01


import aparnik.utils.fields
import django.core.validators
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0004_auto_20180814_0159'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='avatar',
        ),
        migrations.AlterField(
            model_name='user',
            name='username',
            field=aparnik.utils.fields.PhoneField(max_length=30, unique=True, validators=[
                django.core.validators.RegexValidator(code=b'nomatch',
                                                      message='phone is not valid, please insert with code',
                                                      regex=b'^0(?!0)\\d{2}([0-9]{8})$')], verbose_name='Mobile'),
        ),
        migrations.AddField(
            model_name='user',
            name='wallet',
            field=aparnik.utils.fields.PriceField(decimal_places=0, default=0, max_digits=20, verbose_name='Wallet'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db.models.query import Q
from django.shortcuts import get_object_or_404

from rest_framework.generics import ListAPIView, RetrieveUpdateDestroyAPIView
from rest_framework.views import APIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework.response import Response
from rest_framework.exceptions import ValidationError
from rest_framework.serializers import as_serializer_error
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST

from aparnik.contrib.counters.models import Counter

from ..models import BaseModel
from .serializers import ModelListPolymorphicSerializer, ModelDetailsPolymorphicSerializer


class BaseModelSortAPIView(APIView):
    permission_classes = [AllowAny]
    command_dict = {}

    def __init__(self, *args, **kwargs):
        super(BaseModelSortAPIView, self).__init__(*args, **kwargs)
        self.command_dict = {
            'id': {
                'label': 'شناسه',
                'queryset_filter': Q(),
                'annotate_command': {},
                'key_sort': 'id',
            },
            'review_count': {
                'label': 'تعداد نظرات',
                'queryset_filter': Q(),
                'annotate_command': {},
                'key_sort': 'review_count',
            },
            'qa_count': {
                'label': 'تعداد پرسش و پاسخ',
                'queryset_filter': Q(),
                'annotate_command': {},
                'key_sort': 'qa_count',
            }
        }

    def get(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}
        try:
            content = self.get_sort_content_list()
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception as e:
            raise ValidationError(as_serializer_error(e))

    def get_sort_content_list(self):
        return [{'key': key, 'label': value['label']} for key, value in list(self.command_dict.items())]

    def isAllowKey(self, key):
        return key in self.command_dict

    def get_query_sort(self, queryset):

        ordering = None

        if 'ordering' in self.request.query_params:
            ordering = self.request.query_params['ordering']
        else:
            return queryset

        if not ordering and self.isAllowKey(ordering.lstrip('-')):
            return queryset

        command = self.command_dict[ordering.lstrip('-')]

        if ordering[0] == '-':
            command['key_sort'] = '-' + command['key_sort']

        return queryset.filter(command['queryset_filter']).annotate(**command['annotate_command']).order_by(
            command['key_sort'])


class BaseModelListAPIView(ListAPIView):
    serializer_class = ModelListPolymorphicSerializer
    permission_classes = [AllowAny]
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)

    def get_queryset(self):
        return BaseModel.objects.active()


class BaseModelDetailAPIView(RetrieveUpdateDestroyAPIView):
    serializer_class = ModelDetailsPolymorphicSerializer
    queryset = BaseModel.objects.active()
    permission_classes = [AllowAny]
    lookup_url_kwarg = 'model_id'
    lookup_field = 'id'

    def get(self, request, *args, **kwargs):
        model = get_object_or_404(BaseModel.objects.active(), id=kwargs['model_id'])

        if self.request.user.is_anonymous:
            counter = Counter.objects.create(model_obj=model, action='v')
        else:
            counter = Counter.objects.create(user_obj=self.request.user, model_obj=model, action='v')

        counter.save()
        return RetrieveUpdateDestroyAPIView.get(self, request, *args, **kwargs)

    def put(self, request, *args, **kwargs):
        model = get_object_or_404(BaseModel.objects.active(), id=kwargs['model_id'])
        data = request.data.copy()
        # data._mutable = True
        data['resourcetype'] = model.get_real_instance().resourcetype
        serializer = ModelDetailsPolymorphicSerializer(model, data=data, context={'request': request})
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)
        return Response(serializer.errors, status=HTTP_400_BAD_REQUEST)


class TagDetailsAPIView(ListAPIView):
    serializer_class = ModelListPolymorphicSerializer
    permission_classes = [AllowAny]

    # filter_backends = (filters.SearchFilter,)

    def get_queryset(self):
        return BaseModel.objects.active().filter(tags__id=self.kwargs['model_id'])


# class TagDetailsListAPIView(ListAPIView):
#     serializer_class = ModelListPolymorphicSerializer
#     permission_classes = [AllowAny]
#     #filter_backends = (filters.SearchFilter,)
# search_fields = ('id',)
#
# def get_queryset(self):
#     return BaseModel.objects.all()

class ShareAPIView(APIView):
    status = HTTP_400_BAD_REQUEST
    permission_classes = [AllowAny]

    def get(self, request, model_id, *args, **kwargs):

        model = get_object_or_404(BaseModel.objects.all(), id=model_id)

        try:
            status = HTTP_200_OK
            content = 'The content of share uri link.'
            return Response(content, status=status)

        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        return Response(content, status=status)



# -*- coding: utf-8 -*-


from django.contrib import admin

from .models import TermsAndConditions
from aparnik.contrib.basemodels.admin import BaseModelAdmin
# from socials.models import SocialNetwork

# Register your models here.


# class InlineSocialNetwork(admin.StackedInline):
#     extra = 0
#     model = SocialNetwork


class TermsAndConditionsAdmin(BaseModelAdmin):
    fields = ['short_blogs', 'is_active', ]
    list_display = ['is_active', ]
    # inlines = [InlineSocialNetwork]
    list_filter = ['is_active', 'created_at']
    search_fields = []
    exclude = []
    raw_id_fields = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = TermsAndConditionsAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields

    class Meta:
        model = TermsAndConditions


admin.site.register(TermsAndConditions, TermsAndConditionsAdmin)



# -*- coding: utf-8 -*-


from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer, BaseModelListSerializer
from aparnik.contrib.users.api.serializers import UserSummaryListSerializer
from aparnik.packages.educations.progresses.models import Progresses, ProgressSummary
# from aparnik.packages.educations.courses.models import File


class ProgressSummaryListSerializer(BaseModelListSerializer):

    def __init__(self, *args, **kwargs):
        super(ProgressSummaryListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = ProgressSummary
        fields = [
            'percentage',
        ]
        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + ['percentage']


class ProgressListSerializer(BaseModelListSerializer):

    def __init__(self, *args, **kwargs):
        super(ProgressListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Progresses
        fields = BaseModelListSerializer.Meta.fields + [
            'status',
        ]
        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + ['status']


# from aparnik.packages.educations.courses.api.serializers import BaseFileListSerializer
# class BaseProgressListSerializer(BaseModelListSerializer):
#     user = serializers.SerializerMethodField()
#     overall_progress = serializers.SerializerMethodField()
#
#     def __init__(self, *args, **kwargs):
#         super(BaseProgressListSerializer, self).__init__(*args, **kwargs)
#
#     class Meta:
#         model = Progresses
#         fields = BaseModelListSerializer.Meta.fields + [
#             'user',
#             'overall_progress',
#         ]
#         read_only_fields = BaseModelListSerializer.Meta.read_only_fields + []
#
#     def get_user(self, obj):
#         return UserSummaryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data
#
#     def get_overall_progress(self, obj):
#         return Progresses.objects.get_overall_progress(obj.user_obj)
#
#
# class BaseProgressDetailsSerializer(BaseProgressListSerializer, BaseModelDetailSerializer):
#     course = serializers.SerializerMethodField()
#
#     def __init__(self, *args, **kwargs):
#         super(BaseProgressDetailsSerializer, self).__init__(*args, **kwargs)
#
#     class Meta:
#         model = BaseProgresses
#         fields = BaseModelDetailSerializer.Meta.fields + BaseProgressListSerializer.Meta.fields + []
#         read_only_fields = BaseProgressListSerializer.Meta.read_only_fields + BaseModelDetailSerializer.Meta.read_only_fields + []
#
#
# class ProgressListSerializer(BaseProgressListSerializer):
#
#     class Meta:
#         model = Progresses
#         fields = BaseProgressListSerializer.Meta.fields + []
#
#
# class ProgressDetailsSerilizer(ProgressListSerializer, BaseProgressDetailsSerializer):
#     # file = serializers.SerializerMethodField()
#     #
#     # file_obj_id = serializers.IntegerField(required=True, write_only=True)
#
#     def __init__(self, *args, **kwargs):
#         super(ProgressDetailsSerilizer, self).__init__(*args, **kwargs)
#
#     class Meta:
#         model = Progresses
#         fields = ProgressListSerializer.Meta.fields + BaseProgressDetailsSerializer.Meta.fields + [
#             'file_obj',
#             # 'file_obj_id',
#             'status',
#             'is_manual',
#         ]
#
#         validators = []
#
#     # def get_file(self, obj):
#     #     return BaseFileListSerializer(obj.file_obj, many=False, read_only=True,
#     #                                   context=self.context).data
#     #
#     # def create(self, validated_data):
#     #     file_obj = File.objects.filter(id=validated_data.pop('file_obj_id')).first()
#     #     progress = Progresses.objects.create(file_obj=file_obj, **validated_data)
#     #     return progress
#     #
#     # def update(self, instance, validated_data):
#     #     instance.file_obj = File.objects.filter(id=validated_data.get('file_obj_id', None)).get()
#     #     instance.status = validated_data.get('status', None)
#     #     instance.is_manual = validated_data.get('is_manual', None)
#     #     instance.save()
#     #
#     #     return instance
#

# Polymorphic
class BaseProgressListPolymorphicSerializer(PolymorphicSerializer):

    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            # BaseProgresses: BaseProgressListSerializer,
            # Progresses: ProgressListSerializer,
        }
        super(BaseProgressListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


class BaseProgressDetailsPolymorphicSerializer(PolymorphicSerializer):

    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            # BaseProgresses: BaseProgressDetailsSerializer,
            # Progresses: ProgressDetailsSerilizer,
        }
        super(BaseProgressDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}



# -*- coding: utf-8 -*-


from django.db.models import Count
from django.utils.translation import ugettext as _
from rest_framework import filters
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework.serializers import as_serializer_error
from rest_framework.validators import ValidationError

from .serializers import QAListSerializer, QADetailSerializer, QACreateSerializer
from ..models import QA
from aparnik.contrib.reviews.api.serializers import BaseReviewListPolymorphicSerializer, BaseReviewDetailsPolymorphicSerializer


qa_sort_content = [
    {
        'key': 'visit_count',
        'label': 'نمایش'
    },{
        'key': 'review_count',
        'label': 'تعداد نظرات'
    },{
        'key': 'like_count',
        'label': 'محبوب ترین ها'
    }
]


class QASortAPIView(APIView):
    permission_classes = [AllowAny]

    """
        A view that returns the count of active users in JSON.
        """

    def get(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}
        try:
            content = qa_sort_content
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception as e:
            raise ValidationError(as_serializer_error(e))


class QAListAPIView(ListAPIView):
    serializer_class = BaseReviewListPolymorphicSerializer
    permission_classes = [IsAuthenticated]
    # filter_backends = [filters.OrderingFilter, ]
    # ordering_fields = ['visit_count', 'like_count', 'dislike_count', ]
    # ordering = ['visit_count', ]
    search_fields = ('title',)

    def get_queryset(self):
        user = self.request.user
        queryset = QA.objects.all()
        if 'model_id' in self.request.parser_context['kwargs']:
            dict = {
                'model_obj': self.request.parser_context['kwargs']['model_id'],
                'user_obj': None
            }
            if user.is_authenticated:
                dict['user_obj'] = user
            queryset = QA.objects.model_question_answer(**dict)
            # queryset
        if user.is_authenticated:
            queryset = QA.objects.get_this_user(user=user)

        # find order
        allow_ordering = [x['key'] for x in qa_sort_content]
        if 'ordering' in self.request.query_params:
            ordering = self.request.query_params['ordering']
            ordering_field = ordering.lstrip('-')
            if ordering_field in allow_ordering:
                if ordering_field == 'like_count':
                    # TODO: move like count and etc to management base reviews
                    queryset = queryset.annotate(like_count=Count('like'))
                elif ordering_field == 'review_count':
                    queryset = queryset.annotate(review_count=Count('review_obj'))
                queryset = queryset.order_by(ordering)
        return queryset


class QADetailAPIView(RetrieveAPIView):
    serializer_class = BaseReviewDetailsPolymorphicSerializer
    queryset = QA.objects.all()
    permission_classes = [AllowAny]
    lookup_url_kwarg = 'qa_id'
    lookup_field = 'id'


class QACreateAPIView(CreateAPIView):
    serializer_class = QACreateSerializer
    permission_classes = [IsAuthenticated]

    def perform_create(self, serializer):
        user = self.request.user
        files = self.request.POST.get('files', None)
        if files:
            files = files.split(',')
        dict = {
            'user_obj': user,
            'files': []
        }

        if files:
            dict['files'] = files
        serializer.save(**dict)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-10 15:11


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0004_auto_20181210_1509'),
    ]

    operations = [
        migrations.AlterField(
            model_name='news',
            name='cover_images',
            field=models.ManyToManyField(blank=True, to='filefields.FileField', verbose_name='Cover Images'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.contrib.users.admin import get_user_search_fields
from .models import BankName, BankAccount


# Register your models here.
class BankNameAdmin(BaseModelAdmin):
    fields = ['title', 'is_active']
    list_display = ['title', 'is_active']
    list_filter = ['is_active']
    search_fields = ['title']
    exclude = []
    raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = BankNameAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = BankName


class BankAccountAdmin(BaseModelAdmin):
    fields = ['user_obj', 'bank_name_obj', 'account_number', 'card_number', 'shaba_number', 'is_default', 'is_active']
    list_display = ['user_obj', 'bank_name_obj', 'account_number', 'is_default', 'is_active', 'card_number']
    list_filter = ['is_default', 'is_active']
    search_fields = get_user_search_fields('user_obj') + ['bank_name_obj__title', 'account_number', 'shaba_number', 'card_number']
    exclude = []
    raw_id_fields = ['user_obj', 'bank_name_obj']

    def __init__(self, *args, **kwargs):
        Klass = BankAccountAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = BankAccount


admin.site.register(BankName, BankNameAdmin)
admin.site.register(BankAccount, BankAccountAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-13 18:45


from django.db import migrations


def create_through_relations(apps, schema_editor):
    Segment = apps.get_model('segments', 'BaseSegment')
    PageSort = apps.get_model('segments', 'PageSort')
    for obj in Segment.objects.all():
        for page_obj in obj.pages.all():
            PageSort(
                page_obj=page_obj,
                segment_obj=obj,
                sort=0
            ).save()


class Migration(migrations.Migration):

    dependencies = [
        ('segments', '0007_pagesort'),
    ]

    operations = [
        migrations.RunPython(create_through_relations, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-25 11:58


from django.db import migrations


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')
    key = ''
    try:
        key = 'APPROVE_REVIEW'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='منتشر شدن پیام ها به صورت پیش فرض',
            key=key,
            value='False',
            value_type='b',
            is_show=False,
            is_variable_in_home=False,
        )

    try:
        key = 'USER_DEFAULT_LIMIT_DEVICE_LOGIN'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='تعداد گوشی های مجاز هر کاربر',
            key=key,
            value='1',
            value_type='i',
            is_show=False,
            is_variable_in_home=False,
        )

    try:
        key = 'USER_IS_LIMIT_DEVICE_LOGIN_ACTIVE'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='فعال بودن محدودیت دستگاه',
            key=key,
            value='False',
            value_type='b',
            is_show=False,
            is_variable_in_home=False,
        )

    try:
        key = 'MAX_PRODUCT_SHARING_USER_ALLOWED_PER_PRODUCT'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='تعداد دسترسی های داده شده توسط این کاربر برای یک کالای خاص',
            key=key,
            value='1',
            value_type='i',
            is_show=False,
            is_variable_in_home=False,
        )

    try:
        key = 'MAX_PRODUCT_SHARING_USER_ALLOWED_FOR_ALL_PRODUCTS'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='تعداد دسترسی های داده شده توسط این کاربر برای همه کالاها',
            key=key,
            value='10',
            value_type='i',
            is_show=False,
            is_variable_in_home=False,
        )


def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')
    try:
        key = 'APPROVE_REVIEW'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

    try:
        key = 'USER_DEFAULT_LIMIT_DEVICE_LOGIN'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

    try:
        key = 'USER_IS_LIMIT_DEVICE_LOGIN_ACTIVE'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

    try:
        key = 'MAX_PRODUCT_SHARING_USER_ALLOWED_PER_PRODUCT'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

    try:
        key = 'MAX_PRODUCT_SHARING_USER_ALLOWED_FOR_ALL_PRODUCTS'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0018_auto_20190429_1604'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



from rest_framework import serializers

from aparnik.settings import aparnik_settings
from aparnik.contrib.basemodels.api.serializers import BaseModelListSerializer, BaseModelDetailSerializer, ModelListPolymorphicSerializer
from ..models import NotificationForSingleUser

UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


class NotificationListSerializer(BaseModelListSerializer):

    url_read = serializers.SerializerMethodField()
    notification_type = serializers.SerializerMethodField()
    is_read = serializers.SerializerMethodField()
    model = serializers.SerializerMethodField()
    from_user = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(NotificationListSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
            # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = NotificationForSingleUser
        fields = BaseModelListSerializer.Meta.fields + [
            'url_read',
            'title',
            'description',
            'notification_type',
            'datetime',
            'is_read',
            'model',
            'from_user',
        ]

    def get_notification_type(self, obj):
        return obj.get_notification_type_json_display()

    def get_is_read(self, obj):
        return obj.get_user_is_read(user=self.context['request'].user)

    def get_url_read(self, obj):

        return self.context['request'].build_absolute_uri(obj.get_api_read_uri())

    def get_model(self, obj):
        return ModelListPolymorphicSerializer(obj.model_obj, many=False, read_only=True, context=self.context).data if obj.model_obj else None

    def get_from_user(self, obj):
        return UserSummeryListSerializer(obj.from_user_obj, many=False, read_only=True, context=self.context).data if obj.from_user_obj else None


class NotificationDetailSerializer(BaseModelDetailSerializer, NotificationListSerializer):

    class Meta:
        model = NotificationForSingleUser
        fields = BaseModelDetailSerializer.Meta.fields + NotificationListSerializer.Meta.fields + [

        ]



# -*- coding: utf-8 -*-


from django.db import models
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.utils.translation import ugettext as _
from django.core.validators import ValidationError
# from polymorphic.models import PolymorphicModel, PolymorphicManager
from aparnik.contrib.basemodels.models import BaseModel

User = get_user_model()


# Bookmark MANAGER
class BookmarkManager(models.Manager):

    def get_queryset(self):
        return super(BookmarkManager, self).get_queryset()

    def active(self, user=None):
        return self.get_queryset()

    def models(self, model):
        return self.get_queryset().filter(model_obj=model)

    def get_this_user(self, user):
        if not user.is_authenticated:
            return Bookmark.objects.none()
        return self.get_queryset().filter(user_obj=user)


# Bookmark MODEL
class Bookmark(models.Model):
    user_obj = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))
    model_obj = models.ForeignKey(BaseModel, related_name='bookmark_obj', on_delete=models.CASCADE, verbose_name=_('Model'))

    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = BookmarkManager()

    class Meta:
        verbose_name = _('Bookmark')
        verbose_name_plural = _('Bookmarks')

    def __str__(self):
        return "%s" % self.id

    def clean(self):
        bookmark = Bookmark.objects.models(model=self.model_obj).filter(user_obj=self.user_obj)

        if bookmark.count() == 1 and self.id != bookmark.first().id:
            raise ValidationError(
                {'user_obj': [_('You can\'t Bookmark again.')], 'model_obj': [_('You can\'t Bookmark again')]})

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(Bookmark, self).save(*args, **kwargs)

    def get_api_uri(self):
        return reverse('aparnik-api:bookmarks:detail', args=[self.id])

    # def get_api_like_uri(self):
    #     return reverse('api:qa:like:list', args=[self.id])



from django.conf.urls import url, include

from .views import PaymentPayListAPIView, PaymentPayAPIView, PayAPIView, PaymentPayCancelAPIView


app_name = 'payments'

urlpatterns = [
    url(r'^$', PaymentPayListAPIView.as_view(), name='list'),
    url(r'^(?P<uuid>[0-9a-f-]+)/pay/$', PaymentPayAPIView.as_view(), name='payment'),
    url(r'^(?P<uuid>[0-9a-f-]+)/cancel/$', PaymentPayCancelAPIView.as_view(), name='cancel'),
]